@@ -15,12 +15,26 @@
 #   
 #   You should have received a copy of the GNU General Public License
 #   along with this program; if not, write to the Free Software
-#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+#   Foundation, Inc.,  51 Franklin Street, Fifth Floor, Boston, MA
+#   02110-1301 USA
 #
 ###################################################################
 
 .onLoad <- function(dir, package) {
   library.dynam("igraph", package, dir, local=FALSE);
+
+  ########################
+  # Set default parameters
+  ########################
+
+  # printing attributes
+  igraph.par("print.graph.attributes", FALSE)
+  igraph.par("print.vertex.attributes", FALSE)
+  igraph.par("print.edge.attributes", FALSE)
+
+  # verbosity, progress bars mainly
+  igraph.par("verbose", FALSE)
+  
 }
 
 .Last.lib <- function(libpath) {
